//
// Created by Attila Haluska on 8/13/18.
//

#include "compiler/parser/ast/ast_node.h"

#include <stdlib.h>

#include "compiler/lexer/token.h"
#include "memory_management/memory.h"
#include "ast_node.h"

AstNode* ast_node_make_integer(Token* token) {
    AstNode* node = NEW(AstNode);

    char* end;

    node->type = NODE_INTEGER;
    node->token = token;
    node->as.integer = strtoll(node->token->start, &end, 10);

    return node;
}


AstNode* ast_node_make_double(Token* token) {
    AstNode* node = NEW(AstNode);

    char* end;

    node->type = NODE_DOUBLE;
    node->token = token;
    node->as.double_precision = strtold(node->token->start, &end);

    return node;
}

AstNode* ast_node_make_binary_infix_expression(Token* token, AstNode* left, AstNode* right) {
    AstNode* node = NEW(AstNode);

    node->type = NODE_BINARY_INFIX_OPERATOR;
    node->token = token;
    node->as.binary_infix.left = left;
    node->as.binary_infix.right = right;

    return node;
}

AstNode* ast_node_make_unary_prefix_expression(Token* token, AstNode* right) {
    AstNode* node = NEW(AstNode);

    node->type = NODE_UNARY_PREFIX_OPERATOR;
    node->token = token;
    node->as.unary_prefix.right = right;

    return node;
}

static void ast_node_free_integer_node(AstNode* node) {
    token_free(node->token);
    DELETE_TYPE(AstNode, node);
}

static void ast_node_free_double_node(AstNode* node) {
    token_free(node->token);
    DELETE_TYPE(AstNode, node);
}

static void ast_node_free_binary_infix_node(AstNode* node) {
    token_free(node->token);
    ast_node_free(node->as.binary_infix.left);
    ast_node_free(node->as.binary_infix.right);
    DELETE_TYPE(AstNode, node);
}

static void ast_node_free_unary_prefix_node(AstNode* node) {
    token_free(node->token);
    ast_node_free(node->as.unary_prefix.right);
    DELETE_TYPE(AstNode, node);
}

void ast_node_free(AstNode* node) {
    switch (node->type) {
        case NODE_INTEGER:
            ast_node_free_integer_node(node);
            break;
        case NODE_DOUBLE:
            ast_node_free_double_node(node);
            break;
        case NODE_BINARY_INFIX_OPERATOR:
            ast_node_free_binary_infix_node(node);
            break;
        case NODE_UNARY_PREFIX_OPERATOR:
            ast_node_free_unary_prefix_node(node);
            break;
    }
}
