//
// Created by Attila Haluska on 8/13/18.
//

#ifndef ASPL_C_AST_NODE_H
#define ASPL_C_AST_NODE_H

#include <stdint.h>

#include "forward_declarations.h"

typedef enum ast_node_type_t {
    NODE_INTEGER,
    NODE_DOUBLE,
    NODE_BINARY_INFIX_OPERATOR,
    NODE_UNARY_PREFIX_OPERATOR,
} AstNodeType;

typedef struct ast_node_t {
    AstNodeType type;
    Token* token;
    union {
        struct {
            AstNode* left;
            AstNode* right;
        } binary_infix;
        struct {
            AstNode* right;
        } unary_prefix;
        int64_t integer;
        long double double_precision;
    } as;
} AstNode;

extern AstNode* ast_node_make_integer(Token* token);
extern AstNode* ast_node_make_double(Token* token);
extern AstNode* ast_node_make_binary_infix_expression(Token* token, AstNode* left, AstNode* right);
extern AstNode* ast_node_make_unary_prefix_expression(Token* token, AstNode* right);
extern void ast_node_free(AstNode* node);

#endif //ASPL_C_AST_NODE_H
